package utils

import (
	"context"
	"encoding/hex"
	"fmt"
	"github.com/astaxie/beego"
	"github.com/dgrijalva/jwt-go"
	"github.com/ethereum/go-ethereum/common"
	"github.com/ethereum/go-ethereum/ethclient"
	log "github.com/sirupsen/logrus"
	"regexp"
)

var TokenUriMap map[common.Address]string

func init() {
	TokenUriMap = make(map[common.Address]string, 0)
	nftAddr1 := common.HexToAddress("0x23581767a106ae21c074b2276D25e5C3e136a68b")
	TokenUriMap[nftAddr1] = "https://live---metadata-5covpqijaa-uc.a.run.app/metadata/"
}

func IsAddress(params ...string) bool {
	res := true
	for _, param := range params {
		res = common.IsHexAddress(param)
	}
	return res
}

func HexToAddr(val string) common.Address {
	return common.HexToAddress(val)
}

func IsContractAddr(address string, client *ethclient.Client) bool {
	contractAddress := common.HexToAddress(address)
	bytecode, err := client.CodeAt(context.Background(), contractAddress, nil)
	if err != nil {
		return false
	}
	isContract := len(bytecode) > 0
	return isContract
}

func IsNilString(params ...string) bool {
	for _, param := range params {
		if param == "" {
			return false
		}
	}
	return true
}

func GetHttpRegex() *regexp.Regexp {
	urlPattern := beego.AppConfig.String("httpUrlRegex")
	// 编译正则表达式
	regex, err := regexp.Compile(urlPattern)
	if err != nil {
		return nil
	}
	return regex
}

func HexToInt64(hexStr string) (int64, error) {
	// 去除十六进制字符串的前缀 "0x"
	if len(hexStr) > 2 && hexStr[0:2] == "0x" {
		hexStr = hexStr[2:]
	}

	// 解码十六进制字符串
	bytes, err := hex.DecodeString(hexStr)
	if err != nil {
		return 0, err
	}

	// 将解码后的字节切片转换为 int64 类型
	value := int64(0)
	for _, b := range bytes {
		value = (value << 8) | int64(b)
	}

	return value, nil
}

func CheckToken(tokenString string) bool {
	token, _ := jwt.Parse(tokenString, func(token *jwt.Token) (interface{}, error) {
		if _, ok := token.Method.(*jwt.SigningMethodHMAC); !ok {
			return nil, fmt.Errorf("UnExpected signing method")
		}
		return []byte(beego.AppConfig.String("secret")), nil
	})
	if token == nil {
		return false
	}
	claims, _ := token.Claims.(jwt.MapClaims)
	log.Info("claims:", claims)
	return true
}
